
# THIS FILE IS GENERATED FROM PYCHEMIA SETUP.PY
short_version = '0.20.6.1'
version = '0.20.6.1'
full_version = '0.20.6.1.dev0+Unknown'
git_revision = 'Unknown'
name = 'pychemia'
description = 'Python Framework for Materials Discovery and Design'
url = 'https://materialsdiscovery.github.io/PyChemia'
author = 'Guillermo Avendaño Franco'
email = 'gufranco@mail.wvu.edu'
status = 'Development'
copyright = 'Copyright 2020'
date = 'June 1, 2020'
release = False
if not release:
    version = full_version
